//
//  DataManager.m
//  LocationApp_Sample
//
//  Copyright (c) 2015 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "DataManager.h"

#import <libLocationManagerProbe/LocationManager.h>

#define AppSettingPermissionKey         @"PermissionKey"
#define AppSettingLogUploadCountKey     @"LogUploadCountKey"
#define AppSettingDebugModeKey          @"DebugModeKey"
#define AppSettingClientIDKey           @"ClientIDKey"
#define AppSettingDeviceIDKey           @"DeviceIDKey"
#define AppSettingApiKey                @"ApiKey"
#define AppSettingTokenUrlKey           @"TokenUrlKey "
#define AppSettingLocationUrlKey        @"LocationUrlKey "
#define AppSettingSecretKey             @"SecretKey"
#define AppSettingExtraKey              @"ExtraKey"
#define AppSettingUnsearchableExtraKey  @"UnsearchableExtraKey"
#define AppSettingIsSearchKey           @"IsSearchKey"


/** アプリ設定画面のデフォルト値を定義 */
// 位置をサーバに送信する
#define kDefaultSettingPermission                    (NO)
// 位置蓄積件数
#define kDefaultSettingLogUploadCount                1
// デバッグモード
#define kDefaultSettingDebugMode                     (YES)
// クライアントID
#define kDefaultSettingAppTokenClientID              (nil)
// 秘密鍵
#define kDefaultSettingAppTokenSecret                (nil)
// URL(トークン発行サーバセクション)
#define kDefaultSettingAppTokenUrl                   (nil)
// APIキー
#define kDefaultSettingAppApiKey                     (nil)
// URL(位置蓄積サーバセクション)
#define kDefaultSettingLocationUrl                   (nil)
// 端末ID
#define kDefaultSettingDeviceID                      (nil)
// 付加情報
#define kDefaultSettingExtra                         (nil)
// 非検索付加情報
#define kDefaultSettingUnsearchableExtra             (nil)
// 検索DB可能フラグ
#define kDefaultSettingIsSearch                      (YES)

/** ライブラリ設定画面のデフォルト値を定義 */
// APIキー
#define kDefaultSettingLibApiKey                     (nil)
// クライアントID
#define kDefaultSettingLibTokenClientID              (nil)
// 秘密鍵
#define kDefaultSettingLibTokenSecret                (nil)
// 認証サーバ環境
#define kDefaultSettingLibAuthEnv                    0
// 測位精度(m)
#define kDefaultSettingAccuracy                      0.0f
// 距離(m)
#define kDefaultSettingDistance                      0.0f
// バッテリー残量
#define kDefaultSettingBatteryLevInterval            40.0f
// 測位切替精度
#define kDefaultSettingSignificantChangeAccuracy     5000.0f
// 省電力モード
#define kDefaultSettingLowEnergyMode                 (NO)
// 監視領域半径(小)(m)
#define kDefaultSettingMonitoringAreaSmallRadius     300.0
// 監視領域半径(中)(m)
#define kDefaultSettingMonitoringAreaMiddleRadius    1000.0
// 監視領域半径(大)(m)
#define kDefaultSettingMonitoringAreaLargeRadius     2000.0
// 監視領域半径(特大)(m)
#define kDefaultSettingMonitoringAreaOversizeRadius  5000.0
// 監視移動速度(低)(km/h)
#define kDefaultSettingMonitoringSlowSpeed           30.0
// 監視移動速度(中)(km/h)
#define kDefaultSettingMonitoringMiddleSpeed         60.0
// 監視移動速度(高)(km/h)
#define kDefaultSettingMonitoringHighSpeed           100.0


int const IntEmptyValue = INT_MIN;
float const FloatEmptyValue = FLT_MIN;
double const DoubleEmptyValue = DBL_MIN;


@interface DataManager ()
@end

@implementation DataManager

- (instancetype)init
{
    self = [super init];
    if (self != nil)
    {
        self.libAuthEnv = IntEmptyValue;

        self.accuracy = FloatEmptyValue;
        self.distance = FloatEmptyValue;
        self.batteryLevInterval = FloatEmptyValue;
        self.significantChangeAccuracy = FloatEmptyValue;
        self.logUploadCount = IntEmptyValue;
        
        self.monitoringAreaSmallRadius = DoubleEmptyValue;
        self.monitoringAreaMiddleRadius = DoubleEmptyValue;
        self.monitoringAreaLargeRadius = DoubleEmptyValue;
        self.monitoringAreaOversizeRadius = DoubleEmptyValue;
        self.monitoringSlowSpeed = DoubleEmptyValue;
        self.monitoringMiddleSpeed = DoubleEmptyValue;
        self.monitoringHighSpeed = DoubleEmptyValue;
    }
    return self;
}

/* ライブラリ設定値を指定して初期化する */
- (instancetype)initWithLibSettings:(NSDictionary *)settings
{
    self = [self init];
    if (self != nil)
    {
        if (settings[MeasuringSettingApiKey] != nil)
        {
            self.libApiKey = settings[MeasuringSettingApiKey];
        }
        if (settings[MeasuringSettingClientIDKey] != nil)
        {
            self.libTokenClientID = settings[MeasuringSettingClientIDKey];
        }
        if (settings[MeasuringSettingSecretKey] != nil)
        {
            self.libTokenSecret = settings[MeasuringSettingSecretKey];
        }
        if (settings[MeasuringSettingAuthEnvKey] != nil)
        {
            self.libAuthEnv = [settings[MeasuringSettingAuthEnvKey] intValue];
        }
        if (settings[MeasuringSettingAccuracyKey] != nil)
        {
            self.accuracy = [settings[MeasuringSettingAccuracyKey] floatValue];
        }
        if (settings[MeasuringSettingDistanceKey] != nil)
        {
            self.distance = [settings[MeasuringSettingDistanceKey] floatValue];
        }
        if (settings[MeasuringSettingBatteryLevIntervalKey] != nil)
        {
            self.batteryLevInterval = [settings[MeasuringSettingBatteryLevIntervalKey] floatValue];
        }
        if (settings[MeasuringSettingSignificantChangeAccuracyKey] != nil)
        {
            self.significantChangeAccuracy = [settings[MeasuringSettingSignificantChangeAccuracyKey] floatValue];
        }
        if (settings[MeasuringSettingLowEnergyModeKey] != nil)
        {
            self.lowEnergyMode = [settings[MeasuringSettingLowEnergyModeKey] boolValue];
        }
        
        if (settings[MeasuringSettingMonitoringAreaSmallRadiusKey] != nil)
        {
            self.monitoringAreaSmallRadius = [settings[MeasuringSettingMonitoringAreaSmallRadiusKey] doubleValue];
        }
        if (settings[MeasuringSettingMonitoringAreaMiddleRadiusKey] != nil)
        {
            self.monitoringAreaMiddleRadius = [settings[MeasuringSettingMonitoringAreaMiddleRadiusKey] doubleValue];
        }
        if (settings[MeasuringSettingMonitoringAreaLargeRadiusKey] != nil)
        {
            self.monitoringAreaLargeRadius = [settings[MeasuringSettingMonitoringAreaLargeRadiusKey] doubleValue];
        }
        if (settings[MeasuringSettingMonitoringAreaOversizeRadiusKey] != nil)
        {
            self.monitoringAreaOversizeRadius = [settings[MeasuringSettingMonitoringAreaOversizeRadiusKey] doubleValue];
        }
        if (settings[MeasuringSettingMonitoringSlowSpeedKey] != nil)
        {
            self.monitoringSlowSpeed = [settings[MeasuringSettingMonitoringSlowSpeedKey] doubleValue];
        }
        if (settings[MeasuringSettingMonitoringMiddleSpeedKey] != nil)
        {
            self.monitoringMiddleSpeed = [settings[MeasuringSettingMonitoringMiddleSpeedKey] doubleValue];
        }
        if (settings[MeasuringSettingMonitoringHighSpeedKey] != nil)
        {
            self.monitoringHighSpeed = [settings[MeasuringSettingMonitoringHighSpeedKey] doubleValue];
        }
    }
    return self;
}

#pragma Public

/* アプリ設定値を返す */
- (NSDictionary *)appSettings
{
    NSMutableDictionary *settings = [NSMutableDictionary new];

    settings[AppSettingPermissionKey] = @(self.permission);
    if (self.logUploadCount != IntEmptyValue)
    {
        settings[AppSettingLogUploadCountKey] = @(self.logUploadCount);
    }
    settings[AppSettingDebugModeKey] = @(self.debugMode);
    if (self.appTokenClientID != nil)
    {
        settings[AppSettingClientIDKey] = self.appTokenClientID;
    }
    if (self.appTokenSecret != nil)
    {
        settings[AppSettingSecretKey] = self.appTokenSecret;
    }
    if (self.tokenUrl != nil)
    {
        settings[AppSettingTokenUrlKey] = self.tokenUrl;
    }
    if (self.appApiKey != nil)
    {
        settings[AppSettingApiKey] = self.appApiKey;
    }
    if (self.locationUrl != nil)
    {
        settings[AppSettingLocationUrlKey] = self.locationUrl;
    }
    if (self.deviceID != nil)
    {
        settings[AppSettingDeviceIDKey] = self.deviceID;
    }
    if (self.extra != nil)
    {
        settings[AppSettingExtraKey] = self.extra;
    }
    if (self.unsearchableExtra != nil)
    {
        settings[AppSettingUnsearchableExtraKey] = self.unsearchableExtra;
    }
    settings[AppSettingIsSearchKey] = @(self.isSearch);

    return settings;
}

/* 設定値を返す */
- (NSDictionary *)libSettings
{
    NSMutableDictionary *settings = [NSMutableDictionary new];
    
    if (self.libApiKey != nil)
    {
        settings[MeasuringSettingApiKey] = self.libApiKey;
    }
    if (self.libTokenClientID != nil)
    {
        settings[MeasuringSettingClientIDKey] = self.libTokenClientID;
    }
    if (self.libTokenSecret != nil)
    {
        settings[MeasuringSettingSecretKey] = self.libTokenSecret;
    }
    if (self.libAuthEnv != IntEmptyValue)
    {
        settings[MeasuringSettingAuthEnvKey] = @(self.libAuthEnv);
    }
    if (self.accuracy != FloatEmptyValue)
    {
        settings[MeasuringSettingAccuracyKey] = @(self.accuracy);
    }
    if (self.distance != FloatEmptyValue)
    {
        settings[MeasuringSettingDistanceKey] = @(self.distance);
    }
    if (self.batteryLevInterval != FloatEmptyValue)
    {
        settings[MeasuringSettingBatteryLevIntervalKey] = @(self.batteryLevInterval);
    }
    if (self.significantChangeAccuracy != FloatEmptyValue)
    {
        settings[MeasuringSettingSignificantChangeAccuracyKey] = @(self.significantChangeAccuracy);
    }
    settings[MeasuringSettingLowEnergyModeKey] = @(self.lowEnergyMode);
    if (self.monitoringAreaSmallRadius != DoubleEmptyValue)
    {
        settings[MeasuringSettingMonitoringAreaSmallRadiusKey] = @(self.monitoringAreaSmallRadius);
    }
    if (self.monitoringAreaMiddleRadius != DoubleEmptyValue)
    {
        settings[MeasuringSettingMonitoringAreaMiddleRadiusKey] = @(self.monitoringAreaMiddleRadius);
    }
    if (self.monitoringAreaLargeRadius != DoubleEmptyValue)
    {
        settings[MeasuringSettingMonitoringAreaLargeRadiusKey] = @(self.monitoringAreaLargeRadius);
    }
    if (self.monitoringAreaOversizeRadius != DoubleEmptyValue)
    {
        settings[MeasuringSettingMonitoringAreaOversizeRadiusKey] = @(self.monitoringAreaOversizeRadius);
    }
    if (self.monitoringSlowSpeed != DoubleEmptyValue)
    {
        settings[MeasuringSettingMonitoringSlowSpeedKey] = @(self.monitoringSlowSpeed);
    }
    if (self.monitoringMiddleSpeed != DoubleEmptyValue)
    {
        settings[MeasuringSettingMonitoringMiddleSpeedKey] = @(self.monitoringMiddleSpeed);
    }
    if (self.monitoringHighSpeed != DoubleEmptyValue)
    {
        settings[MeasuringSettingMonitoringHighSpeedKey] = @(self.monitoringHighSpeed);
    }

    return settings;
}

/* アプリ設定値の保存 */
- (void)saveAppSettings
{
    NSUserDefaults *standardUserDefaults = [NSUserDefaults standardUserDefaults];
    [standardUserDefaults setObject:[self appSettings] forKey:@"AppSettings"];
    [standardUserDefaults synchronize];
}

/* ライブラリ設定値の保存 */
- (void)saveLibSettings
{
    NSUserDefaults *standardUserDefaults = [NSUserDefaults standardUserDefaults];
    [standardUserDefaults setObject:[self libSettings] forKey:@"LibSettings"];
    [standardUserDefaults synchronize];
}

/* 設定値の復元 */
- (void)restoreSettings
{
    [self _restoreAppSettings:NO];
    [self _restoreLibSettings:NO];
}

/* 設定値のリセット */
- (void)resetAppSettings
{
    [self _restoreAppSettings:YES];
}

/* 設定値のリセット */
- (void)resetLibSettings
{
    [self _restoreLibSettings:YES];
}

/* アプリ設定の送信パラメータ必須判定 */
- (BOOL)isRequiredAppSendingParameters
{
    return self.tokenUrl != nil && self.appTokenClientID != nil && self.appTokenSecret != nil
        && self.locationUrl != nil && self.appApiKey != nil;
}

#pragma mark Private

/* アプリ設定値の復元 */
- (BOOL)_restoreAppSettings:(BOOL)reset
{
    NSDictionary *settings = nil;
    BOOL update = NO;
    
    if (reset) {
        update = YES;
    } else {
        NSUserDefaults *standardUserDefaults = [NSUserDefaults standardUserDefaults];
        settings = [standardUserDefaults objectForKey:@"AppSettings"];
        
        id object;
        
        if (settings == nil) {
            reset = YES;
        } else {
            object = [settings objectForKey:AppSettingPermissionKey];
            self.permission = [object boolValue];

            object = [settings objectForKey:AppSettingLogUploadCountKey];
            self.logUploadCount = [object intValue];
 
            object = [settings objectForKey:AppSettingDebugModeKey];
            self.debugMode = [object boolValue];

            object = [settings objectForKey:AppSettingClientIDKey];
            self.appTokenClientID = object;

            object = [settings objectForKey:AppSettingSecretKey];
            self.appTokenSecret = object;

            object = [settings objectForKey:AppSettingTokenUrlKey];
            self.tokenUrl = object;
            
            object = [settings objectForKey:AppSettingApiKey];
            self.appApiKey = object;

            object = [settings objectForKey:AppSettingLocationUrlKey];
            self.locationUrl = object;

            object = [settings objectForKey:AppSettingDeviceIDKey];
            self.deviceID = object;

            object = [settings objectForKey:AppSettingExtraKey];
            self.extra = object;
            
            object = [settings objectForKey:AppSettingUnsearchableExtraKey];
            self.unsearchableExtra = object;
            
            object = [settings objectForKey:AppSettingIsSearchKey];
            self.isSearch = [object boolValue];
        }
    }
    
    if (reset) {
        self.permission = kDefaultSettingPermission;
        self.logUploadCount = kDefaultSettingLogUploadCount;
        self.debugMode = kDefaultSettingDebugMode;
        self.appTokenClientID = kDefaultSettingAppTokenClientID;
        self.appTokenSecret = kDefaultSettingAppTokenSecret;
        self.tokenUrl = kDefaultSettingAppTokenUrl;
        self.appApiKey = kDefaultSettingAppApiKey;
        self.locationUrl = kDefaultSettingLocationUrl;
        self.deviceID = kDefaultSettingDeviceID;
        self.extra = kDefaultSettingExtra;
        self.unsearchableExtra = kDefaultSettingUnsearchableExtra;
        self.isSearch = kDefaultSettingIsSearch;
    }
    
    // メモリで保持している内容を永続化
    [self saveAppSettings];
    
    return update;
}

/* 設定値の復元 */
- (BOOL)_restoreLibSettings:(BOOL)reset
{
    NSDictionary *settings = nil;
    BOOL update = NO;

    if (reset) {
        update = YES;
    } else {
        NSUserDefaults *standardUserDefaults = [NSUserDefaults standardUserDefaults];
        settings = [standardUserDefaults objectForKey:@"LibSettings"];

        id object;

        if (settings == nil) {
            reset = YES;
        } else {
            object = [settings objectForKey:MeasuringSettingApiKey];
            self.libApiKey = object;

            object = [settings objectForKey:MeasuringSettingClientIDKey];
            self.libTokenClientID = object;

            object = [settings objectForKey:MeasuringSettingSecretKey];
            self.libTokenSecret = object;

            object = [settings objectForKey:MeasuringSettingAuthEnvKey];
            self.libAuthEnv = [object intValue];

            object = [settings objectForKey:MeasuringSettingAccuracyKey];
            self.accuracy = [object floatValue];

            object = [settings objectForKey:MeasuringSettingDistanceKey];
            self.distance = [object floatValue];

            object = [settings objectForKey:MeasuringSettingBatteryLevIntervalKey];
            self.batteryLevInterval = [object floatValue];

            object = [settings objectForKey:MeasuringSettingSignificantChangeAccuracyKey];
            self.significantChangeAccuracy = [object floatValue];

            object = [settings objectForKey:MeasuringSettingLowEnergyModeKey];
            self.lowEnergyMode = [object boolValue];

            object = [settings objectForKey:MeasuringSettingMonitoringAreaSmallRadiusKey];
            self.monitoringAreaSmallRadius = [object doubleValue];
            object = [settings objectForKey:MeasuringSettingMonitoringAreaMiddleRadiusKey];
            self.monitoringAreaMiddleRadius = [object doubleValue];

            object = [settings objectForKey:MeasuringSettingMonitoringAreaLargeRadiusKey];
            self.monitoringAreaLargeRadius = [object doubleValue];

            object = [settings objectForKey:MeasuringSettingMonitoringAreaOversizeRadiusKey];
            self.monitoringAreaOversizeRadius = [object doubleValue];

            object = [settings objectForKey:MeasuringSettingMonitoringSlowSpeedKey];
            self.monitoringSlowSpeed = [object doubleValue];

            object = [settings objectForKey:MeasuringSettingMonitoringMiddleSpeedKey];
            self.monitoringMiddleSpeed = [object doubleValue];

            object = [settings objectForKey:MeasuringSettingMonitoringHighSpeedKey];
            self.monitoringHighSpeed = [object doubleValue];
        }
    }
    
    if (reset) {
        self.libApiKey = kDefaultSettingLibApiKey;
        self.libTokenClientID = kDefaultSettingLibTokenClientID;
        self.libTokenSecret = kDefaultSettingLibTokenSecret;
        self.libAuthEnv = kDefaultSettingLibAuthEnv;
        self.accuracy = kDefaultSettingAccuracy;
        self.distance = kDefaultSettingDistance;
        self.batteryLevInterval = kDefaultSettingBatteryLevInterval;
        self.significantChangeAccuracy = kDefaultSettingSignificantChangeAccuracy;
        self.lowEnergyMode = kDefaultSettingLowEnergyMode;
        self.monitoringAreaSmallRadius = kDefaultSettingMonitoringAreaSmallRadius;
        self.monitoringAreaMiddleRadius = kDefaultSettingMonitoringAreaMiddleRadius;
        self.monitoringAreaLargeRadius = kDefaultSettingMonitoringAreaLargeRadius;
        self.monitoringAreaOversizeRadius = kDefaultSettingMonitoringAreaOversizeRadius;
        self.monitoringSlowSpeed = kDefaultSettingMonitoringSlowSpeed;
        self.monitoringMiddleSpeed = kDefaultSettingMonitoringMiddleSpeed;
        self.monitoringHighSpeed = kDefaultSettingMonitoringHighSpeed;
    }
    
    // メモリで保持している内容を永続化
    [self saveLibSettings];
    
    return update;
}

@end
